body {
  font-family: Cambria, Cochin, Georgia, Times, 'Times New Roman', serif;
}

#authSiteHeaderNavBar > .navbar-nav > .nav-item > .nav-link > span {
  color: #212529;
  text-decoration: none;
}

#authSiteHeaderNavBar > .navbar-nav > .nav-item > .nav-link > span:hover {
  color: #212529;
  text-decoration: underline;
}

.requirement, .errormessage {
  color: red;
}

.hover {
  text-decoration: none;
  color: #212529;
}

.hover:hover {
  text-decoration: underline;
  color: #212529;
}
/*# sourceMappingURL=layout.css.map */