body {
    font-family: Cambria, Cochin, Georgia, Times, 'Times New Roman', serif;
}


#authSiteHeaderNavBar > .navbar-nav > .nav-item > .nav-link > span {
    color: $dark;
    text-decoration: none;
    &:hover {
        color: $dark;
        text-decoration: underline;
    }
}

.requirement, .errormessage {
    color: red;
}