{% load static %}
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    {% if FAVICON %}
    <link rel="icon" href="{% static 'authentication/favicon_io/favicon.ico' %}" type = "image/x-icon">
    {% endif %}

    <link href="{% static 'authentication/assets/node_modules/bootstrap/dist/css/bootstrap.css' %}" rel="stylesheet">
    <link href="{% static 'authentication/assets/node_modules/bootstrap-icons/font/bootstrap-icons.css' %}" rel="stylesheet">
    <link href="{% static 'authentication/css/layout.css' %}" rel="stylesheet">
    {% block style %}{% endblock %}
    <title>{% block title %}{% endblock %}</title>
</head>
<body class="p-1 m-1">
    {% include 'authentication/header.html' with request=request SITE_TITLE=SITE_TITLE %}
    {% block body %}{% endblock %}
    <script src="{% static 'authentication/assets/node_modules/bootstrap/dist/js/bootstrap.bundle.js' %}"></script>
    <script src="{% static 'authentication/assets/node_modules/jquery/dist/jquery.js' %}"></script>
    <script src="{% static 'authentication/assets/node_modules/handlebars/dist/handlebars.js' %}"></script>
    <script>
        var tooltipTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="tooltip"]'))
        var tooltipList = tooltipTriggerList.map(function (tooltipTriggerEl) {
            return new bootstrap.Tooltip(tooltipTriggerEl)
        })
    </script>
    <!--Script for popovers-->
    <script>
        var popoverTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="popover"]'))
        var popoverList = popoverTriggerList.map(function (popoverTriggerEl) {
            return new bootstrap.Popover(popoverTriggerEl)
        })
    </script>
    {% block static %}{% endblock %}
</body>
</html>